from src.open_meteo_data_dvl.meteo_bz_client import MeteoBzClient


# List of strings to choose from
scodes = ["Custom", "83200MS", "27100MS", "23200MS", "35100WS"]
snames = ["", "Bolzano", "Gargazon", "Meran", "Jaufenkamm"]

# ask user to change one of the options
print("Choose a station code to get data from:")

for i in range(len(scodes)):
    print(str(i) + ": " + scodes[i], snames[i])

try:
    choice = int(input("Choose a number: "))
except ValueError:
    print("Invalid choice.")
    exit()

# Prompt for custom station code
if choice == 0:
    scode: str = input("Enter a station code: ")

    # Exit if no station code is provided
    if scode == "":
        print("Must provide station code.")
        exit()

# Choice is in range
elif choice >= 0 and choice < len(scodes):
    scode: str = scodes[choice]

# Choice is out of range
else:
    print("Invalid choice.")
    exit()

# Get station data
station = MeteoBzClient().get_station(scode)

if station is not None:
    # Get sensor data
    station.sensors = MeteoBzClient().get_sensors(station)

    # Print data
    print("---")
    print(
        f"{station.name_eng} {station.altitude}m ({station.station_code}) {station.sensors[0].datetime}:"
    )

    for sensor in station.sensors:
        print(f"{sensor.description_deu}: {sensor.value} {sensor.unit} ({sensor.type})")

    # Specific Sensor
    print("---")
    sensorLT = MeteoBzClient().get_sensor(station, "LT")
    print(
        f"{sensorLT.description_deu}: {sensorLT.value} {sensorLT.unit} ({sensorLT.type})"
    )

    sensorGS = MeteoBzClient().get_sensor(station, "GS")
    print(
        f"{sensorGS.description_deu}: {sensorGS.value} {sensorGS.unit} ({sensorGS.type})"
    )



"""Class for a meteo station object"""

from dataclasses import dataclass, field
from .meteo_bz_sensor import MeteoBzSensor
from typing import Optional


@dataclass
class MeteoBzStation:
    altitude: str
    latitude: str
    longitude: str
    name_deu: str
    name_eng: str
    name_ita: str
    name_lld: str
    station_code: str

    sensors: list[MeteoBzSensor] = field(default_factory=list)

    def __init__(self, apidata: dict[str, str]):
        # Set attributes
        self._data_to_attributes(apidata)

    # Convert API data to attributes
    def _data_to_attributes(self, apidata: dict[str, str]):
        self.station_code = apidata["SCODE"]

        # Station description
        self.name_deu = apidata["NAME_D"]
        self.name_eng = apidata["NAME_E"]
        self.name_ita = apidata["NAME_I"]
        self.name_lld = apidata["NAME_L"]

        # Geographical data
        self.altitude = apidata["ALT"]
        self.longitude = apidata["LONG"]
        self.latitude = apidata["LAT"]

    # Get sensor types from station
    @property
    def sensor_types(self) -> list[str]:
        if self.sensors == []:
            return []

        sensor_types: list[str] = []
        for sensor in self.sensors:
            sensor_types.append(str(sensor.type))

        return sensor_types

    # Add a sensor to the station
    def get_sensor(self, type: str) -> Optional[MeteoBzSensor]:
        for sensor in self.sensors:
            if sensor.type == type:
                return sensor



"""Class for a meteo sensor object"""

import datetime
from dataclasses import dataclass
from .const import DATE_FORMAT


@dataclass
class MeteoBzSensor:
    date: str
    description_deu: str
    description_ita: str
    description_lld: str
    station_code: str
    type: str
    unit: str
    value: str

    def __init__(self, apidata: dict[str, str]):

        # Set attributes
        self._data_to_attributes(apidata)

    # Convert API data to attributes
    def _data_to_attributes(self, apidata: dict[str, str]):
        self.station_code = apidata["SCODE"]

        # Sensor description
        self.description_deu = apidata["DESC_D"]
        self.description_ita = apidata["DESC_I"]
        self.description_lld = apidata["DESC_L"]

        # Sensor data
        self.type = apidata["TYPE"]
        self.unit = apidata["UNIT"]
        self.value = apidata["VALUE"]

        # Sensor measurement date
        self.date = apidata["DATE"]
        self.datetime = datetime.datetime.strptime(self.date, DATE_FORMAT)



import requests
from typing import Optional

from .meteo_bz_sensor import MeteoBzSensor
from .meteo_bz_station import MeteoBzStation


class MeteoBzClient:
    stations: list[MeteoBzStation] = []
    sensors: list[MeteoBzSensor] = []

    # Get all stations
    def get_stations(self) -> list[MeteoBzStation]:
        # GET request to the API
        response = requests.get(
            "http://daten.buergernetz.bz.it/services/meteo/v1/stations"
        )

        # Get the data from the response
        data = response.json().get("features")

        # Loop through data and create a station objects
        for feature in data:
            station_data = feature["properties"]
            self.stations.append(MeteoBzStation(station_data))

        return self.stations

    # Get a single station
    def get_station(self, station_code: str) -> Optional[MeteoBzStation]:
        # Get all stations
        self.get_stations()

        # Loop through stations and return the station with the given station_code
        for station in self.stations:
            if station.station_code == station_code:
                return station

    # Get sensors from station
    def get_sensors(self, station: MeteoBzStation) -> list[MeteoBzSensor]:
        sensors: list[MeteoBzSensor] = []

        # GET request to the API
        response = requests.get(
            "http://daten.buergernetz.bz.it/services/meteo/v1/sensors",
            params={"station_code": station.station_code},
        )

        # Get the data from the response
        data = response.json()

        # Loop through data and create a sensor objects
        for sensor_data in data:
            sensors.append(MeteoBzSensor(sensor_data))

        return sensors

    # Get a single sensor from a station
    def get_sensor(self, station: MeteoBzStation, type: str) -> MeteoBzSensor:
        # GET request to the API
        response = requests.get(
            "http://daten.buergernetz.bz.it/services/meteo/v1/sensors",
            params={"station_code": station.station_code, "sensor_code": type},
        )

        data = response.json()
        if not data:
            raise ValueError(
                f"No sensor found with type '{type}' for station '{station.station_code}'"
            )

        return MeteoBzSensor(data[0])
